@@ -17,7 +17,6 @@ from quodlibet.util import is_windows
 from quodlibet.qltk import Icons
 from quodlibet.pattern import Pattern
 from quodlibet.qltk.entry import UndoEntry
-from quodlibet.util import gdecode
 from .util import pconfig
 
 
@@ -102,7 +101,7 @@ class Preferences(Gtk.VBox):
             child.show_all()
 
     def __changed_entry(self, entry, label, frame):
-        text = gdecode(entry.get_text())
+        text = entry.get_text()
 
         if app.player.info is None:
             text = _("Not playing")
