@@ -3,6 +3,10 @@
 #        2016-17 Nick Boultbee
 #           2017 Didier Villevalois
 #           2017 Muges
+#           2017 Eyenseo
+#           2018 Joschua Gandert
+#           2018 Blimmo
+#           2018 Olli Helin
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -47,6 +51,7 @@ class WaveformSeekBar(Gtk.Box):
 
         for child in self.get_children():
             child.show_all()
+        self.set_time_label_visibility(CONFIG.show_time_labels)
 
         self._waveform_scale.connect('size-allocate',
                                      self._update_redraw_interval)
@@ -73,24 +78,38 @@ class WaveformSeekBar(Gtk.Box):
         if player.info:
             self._create_waveform(player.info, CONFIG.max_data_points)
 
+    def set_time_label_visibility(self, is_visible):
+        self._time_labels_visible = is_visible
+        if is_visible:
+            self._elapsed_label.show()
+            self._remaining_label.show()
+        else:
+            self._elapsed_label.hide()
+            self._remaining_label.hide()
+
     def _create_waveform(self, song, points):
         # Close any existing pipeline to avoid leaks
         self._clean_pipeline()
 
+        if not song.is_file:
+            return
+
         command_template = """
-        filesrc name=fs
-        ! decodebin ! audioconvert
+        uridecodebin name=uridec
+        ! audioconvert
         ! level name=audiolevel interval={} post-messages=true
         ! fakesink sync=false"""
         interval = int(song("~#length") * 1E9 / points)
+        if not interval:
+            return
         print_d("Computing data for each %.3f seconds" % (interval / 1E9))
 
         command = command_template.format(interval)
         pipeline = Gst.parse_launch(command)
-        pipeline.get_by_name("fs").set_property("location", song("~filename"))
+        pipeline.get_by_name("uridec").set_property("uri", song("~uri"))
 
         bus = pipeline.get_bus()
-        self._bus_id = bus.connect("message", self._on_bus_message)
+        self._bus_id = bus.connect("message", self._on_bus_message, points)
         bus.add_signal_watch()
 
         pipeline.set_state(Gst.State.PLAYING)
@@ -98,7 +117,8 @@ class WaveformSeekBar(Gtk.Box):
         self._pipeline = pipeline
         self._new_rms_vals = []
 
-    def _on_bus_message(self, bus, message):
+    def _on_bus_message(self, bus, message, points):
+        force_stop = False
         if message.type == Gst.MessageType.ERROR:
             error, debug = message.parse_error()
             print_d("Error received from element {name}: {error}".format(
@@ -114,10 +134,16 @@ class WaveformSeekBar(Gtk.Box):
                     # Normalize dB value to value between 0 and 1
                     rms = pow(10, (rms_db_avg / 20))
                     self._new_rms_vals.append(rms)
+                    if len(self._new_rms_vals) >= points:
+                        # The audio might be much longer than we anticipated
+                        # and we would get way too many events due to the too
+                        # short interval set.
+                        force_stop = True
             else:
                 print_w("Got unexpected message of type {}"
                         .format(message.type))
-        elif message.type == Gst.MessageType.EOS:
+
+        if message.type == Gst.MessageType.EOS or force_stop:
             self._clean_pipeline()
 
             # Update the waveform with the new data
@@ -166,10 +192,6 @@ class WaveformSeekBar(Gtk.Box):
     def _on_song_changed(self, library, songs, player):
         if not player.info:
             return
-        if not player.info.is_file:
-            print_d("%s is not a local file, skipping waveform calculation."
-                    % player.info("~filename"))
-            return
         # Check that the currently playing song has changed
         if player.info in songs:
             # Trigger a re-computation of the waveform
@@ -179,7 +201,7 @@ class WaveformSeekBar(Gtk.Box):
             self._update_label(player)
 
     def _on_song_started(self, player, song):
-        if player.info and player.info.is_file:
+        if player.info:
             # Trigger a re-computation of the waveform
             self._create_waveform(player.info, CONFIG.max_data_points)
             self._resize_labels(player.info)
@@ -195,6 +217,10 @@ class WaveformSeekBar(Gtk.Box):
         self._update_waveform(player, full_redraw)
 
     def _update_label(self, player):
+        if not self._time_labels_visible:
+            self.set_sensitive(player.info is not None and player.seekable)
+            return
+
         if player.info:
             if self._hovering:
                 # Show the position pointed by the mouse
@@ -238,7 +264,12 @@ class WaveformSeekBar(Gtk.Box):
             self._waveform_scale.queue_draw()
 
     def _on_mouse_hover(self, _, event):
-        self._waveform_scale.set_mouse_x_position(event.x)
+        def clamp(a, x, b):
+            '''Return x if a <= x <= b, else the a or b nearest to x.'''
+            return min(max(x, a), b)
+
+        width = self._waveform_scale.get_allocation().width
+        self._waveform_scale.set_mouse_x_position(clamp(0, event.x, width))
 
         if self._hovering:
             (x, y, w, h) = self._waveform_scale.compute_hover_redraw_area()
@@ -294,7 +325,8 @@ class WaveformScale(Gtk.EventBox):
 
         self.mouse_position = -1
         self._last_mouse_position = -1
-        self.add_events(Gdk.EventMask.POINTER_MOTION_MASK)
+        self.add_events(Gdk.EventMask.POINTER_MOTION_MASK |
+                        Gdk.EventMask.SCROLL_MASK)
 
         self._seeking = False
 
@@ -319,6 +351,14 @@ class WaveformScale(Gtk.EventBox):
 
         # Compute the coarsest time interval for redraws
         length = self._player.info("~#length")
+        if (length == 0):
+            # The length is 0 for example when playing a stream from
+            # Internet radio. If 0 is passed forward as the update interval,
+            # UI will freeze as it will try to update continuously.
+            # The update interval is usually 1 second so use that instead.
+            print_d("Length is zero for %s, using redraw interval of 1000 ms"
+                % self._player.info)
+            return 1000
         return length * 1000 / max(width * pixel_ratio, 1)
 
     def compute_redraw_area(self):
@@ -376,23 +416,31 @@ class WaveformScale(Gtk.EventBox):
             for x in range(int(floor(cx * pixel_ratio)),
                            int(ceil((cx + cw) * pixel_ratio)), 1):
 
-                if self._seeking and mouse_position >= 0:
-                    # The user is seeking (holding mousebutton down)
-                    fg_color = (elapsed_color if x < mouse_position
-                                else remaining_color)
-                elif mouse_position >= 0:
-                    # The mouse is hovering the seekbar
-                    fg_color = (hover_color if x < mouse_position
-                                else remaining_color)
+                if mouse_position >= 0:
+                    if self._seeking:
+                        # The user is seeking (holding mousebutton down)
+                        fg_color = (elapsed_color if x < mouse_position
+                                    else remaining_color)
+                    elif show_current_pos_config:
+                        # Use hover color and elapsed color to display the
+                        # current playing position while hovering
+                        if x < mouse_position:
+                            if x < position_width:
+                                fg_color = elapsed_color
+                            else:
+                                fg_color = hover_color
+                        elif x < position_width:
+                            fg_color = hover_color
+                        else:
+                            fg_color = remaining_color
+                    else:
+                        # The mouse is hovering the seekbar
+                        fg_color = (hover_color if x < mouse_position
+                                    else remaining_color)
                 else:
                     fg_color = (elapsed_color if x < position_width
                                 else remaining_color)
 
-                # Draw a line of width scale_factor at the current position
-                if show_current_pos_config:
-                    if position_width - scale_factor <= x < position_width:
-                        fg_color = elapsed_color
-
                 cr.set_source_rgba(*list(fg_color))
 
                 # Basic anti-aliasing / oversampling
@@ -512,6 +560,14 @@ class WaveformScale(Gtk.EventBox):
             self.queue_draw()
             return True
 
+    def do_scroll_event(self, event):
+        if event.direction == Gdk.ScrollDirection.UP:
+            self._player.seek(self._player.get_position() + CONFIG.seek_amount)
+            self.queue_draw()
+        elif event.direction == Gdk.ScrollDirection.DOWN:
+            self._player.seek(self._player.get_position() - CONFIG.seek_amount)
+            self.queue_draw()
+
     def set_position(self, position):
         self.position = position
 
@@ -533,7 +589,10 @@ class Config(object):
     hover_color = ConfProp(_config, "hover_color", "")
     remaining_color = ConfProp(_config, "remaining_color", "")
     show_current_pos = BoolConfProp(_config, "show_current_pos", False)
+    seek_amount = IntConfProp(_config, "seek_amount", 5000)
     max_data_points = IntConfProp(_config, "max_data_points", 3000)
+    show_time_labels = BoolConfProp(_config, "show_time_labels", True)
+
 
 CONFIG = Config()
 
@@ -548,6 +607,9 @@ class WaveformSeekBarPlugin(EventPlugin):
     PLUGIN_DESC = _(
         "A seekbar in the shape of the waveform of the current song.")
 
+    def __init__(self):
+        self._bar = None
+
     def enabled(self):
         self._bar = WaveformSeekBar(app.player, app.librarian)
         self._bar.show()
@@ -556,7 +618,7 @@ class WaveformSeekBarPlugin(EventPlugin):
     def disabled(self):
         app.window.set_seekbar_widget(None)
         self._bar.destroy()
-        del self._bar
+        self._bar = None
 
     def PluginPreferences(self, parent):
         red = Gdk.RGBA()
@@ -589,8 +651,17 @@ class WaveformSeekBarPlugin(EventPlugin):
 
         def on_show_pos_toggled(button, *args):
             CONFIG.show_current_pos = button.get_active()
+
+        def seek_amount_changed(spinbox):
+            CONFIG.seek_amount = spinbox.get_value_as_int()
+
         vbox = Gtk.VBox(spacing=6)
 
+        def on_show_time_labels_toggled(button, *args):
+            CONFIG.show_time_labels = button.get_active()
+            if self._bar is not None:
+                self._bar.set_time_label_visibility(CONFIG.show_time_labels)
+
         def create_color(label_text, color, callback):
             hbox = Gtk.HBox(spacing=6)
             hbox.set_border_width(6)
@@ -620,4 +691,24 @@ class WaveformSeekBarPlugin(EventPlugin):
         show_current_pos.connect("toggled", on_show_pos_toggled)
         vbox.pack_start(show_current_pos, True, True, 0)
 
+        show_time_labels = Gtk.CheckButton(label=_("Show time labels"))
+        show_time_labels.set_active(CONFIG.show_time_labels)
+        show_time_labels.connect("toggled", on_show_time_labels_toggled)
+        vbox.pack_start(show_time_labels, True, True, 0)
+
+        hbox = Gtk.HBox(spacing=6)
+        hbox.set_border_width(6)
+        label = Gtk.Label(label=_(
+            "Seek amount when scrolling (milliseconds):"
+        ))
+        hbox.pack_start(label, False, True, 0)
+        seek_amount = Gtk.SpinButton(
+            adjustment=Gtk.Adjustment(CONFIG.seek_amount,
+                                      0, 60000, 1000, 1000, 0)
+        )
+        seek_amount.set_numeric(True)
+        seek_amount.connect("changed", seek_amount_changed)
+        hbox.pack_start(seek_amount, True, True, 0)
+        vbox.pack_start(hbox, True, True, 0)
+
         return vbox
