@@ -35,36 +35,15 @@ class BigCenteredImage(qltk.Window):
         parent = qltk.get_top_parent(parent)
         self.set_transient_for(parent)
 
-        if qltk.is_wayland():
-            # no screen size with wayland, the parent window is
-            # the next best thing..
-            width, height = parent.get_size()
-            width = int(width / 1.1)
-            height = int(height / 1.1)
-        else:
-            width = int(Gdk.Screen.width() / 1.8)
-            height = int(Gdk.Screen.height() / 1.8)
-
         self.set_position(Gtk.WindowPosition.CENTER_ON_PARENT)
 
-        scale_factor = self.get_scale_factor()
-
-        pixbuf = None
-        try:
-            pixbuf = pixbuf_from_file(fileobj, (width, height), scale_factor)
-        except GLib.GError:
-            pass
-
-        # failed to load, abort
-        if not pixbuf:
+        #If image fails to set, abort construction.
+        if not self.set_image(fileobj, parent):
             self.destroy()
             return
 
-        image = Gtk.Image()
-        image.set_from_surface(get_surface_for_pixbuf(self, pixbuf))
-
         event_box = Gtk.EventBox()
-        event_box.add(image)
+        event_box.add(self.__image)
 
         frame = Gtk.Frame()
         frame.set_shadow_type(Gtk.ShadowType.OUT)
@@ -74,8 +53,58 @@ class BigCenteredImage(qltk.Window):
 
         event_box.connect('button-press-event', self.__destroy)
         event_box.connect('key-press-event', self.__destroy)
+
         self.get_child().show_all()
 
+    def set_image(self, file, parent):
+        scale_factor = self.get_scale_factor()
+
+        (width, height) = self.__calculate_screen_width(parent)
+
+        pixbuf = None
+        try:
+            pixbuf = pixbuf_from_file(file, (width, height), scale_factor)
+        except GLib.GError:
+            return False
+
+        # failed to load, abort
+        if not pixbuf:
+            return False
+
+        self.__image = Gtk.Image()
+        self.__image.set_from_surface(get_surface_for_pixbuf(self, pixbuf))
+
+        return True
+
+    def __calculate_screen_width(self, parent):
+        if qltk.is_wayland():
+            # no screen size with wayland, the parent window is
+            # the next best thing..
+            width, height = parent.get_size()
+            width = int(width / 1.1)
+            height = int(height / 1.1)
+            return (width, height)
+        else:
+            win = parent.get_window()
+            if win:
+                if qltk.gtk_version[:2] >= (3, 22):
+                    disp = Gdk.Display.get_default()
+                    mon = disp.get_monitor_at_window(win)
+                    rect = mon.get_geometry()
+                else:
+                    # The result should be the same as above, just using
+                    # deprecated methods instead
+                    screen = Gdk.Screen.get_default()
+                    mon_num = screen.get_monitor_at_window(win)
+                    rect = screen.get_monitor_geometry(mon_num)
+                width = int(rect.width / 1.8)
+                height = int(rect.height / 1.8)
+                return (width, height)
+
+            width = int(Gdk.Screen.width() / 1.8)
+            height = int(Gdk.Screen.height() / 1.8)
+            return (width, height)
+
     def __destroy(self, *args):
         self.destroy()
 
@@ -226,7 +255,7 @@ class CoverImage(Gtk.EventBox):
         self.__cancellable = None
 
         self.add(ResizeImage(resize, size))
-        self.connect('button-press-event', self.__show_cover)
+        self.connect('button-press-event', self.__album_clicked)
         self.set_song(song)
         self.get_child().show_all()
 
@@ -249,41 +278,54 @@ class CoverImage(Gtk.EventBox):
                     try:
                         self.set_image(result)
                         self.emit('cover-visible', success)
+                        self.update_bci(result)
                         # If this widget is already 'destroyed', we will get
                         # following error.
                     except AttributeError:
                         pass
+                else:
+                    self.update_bci(None)
             app.cover_manager.acquire_cover(cb, cancellable, song)
 
     def refresh(self):
         self.set_song(self.__song)
 
+    def update_bci(self, albumfile):
+        #if there's a big image displaying, it should update.
+        if self.__current_bci is not None:
+            self.__current_bci.destroy()
+            if albumfile:
+                self.__show_cover(self.__song)
+
     def __nonzero__(self):
         return bool(self.__file)
 
     def __reset_bci(self, bci):
         self.__current_bci = None
 
-    def __show_cover(self, box, event):
-        """Show the cover as a detached BigCenteredImage.
-        If one is already showing, destroy it instead
-        If there is no image, run the AlbumArt plugin
-        """
-
+    def __album_clicked(self, box, event):
         song = self.__song
         if not song:
             return
 
         if event.button != Gdk.BUTTON_PRIMARY or \
                 event.type != Gdk.EventType.BUTTON_PRESS:
-            return
+            return False
 
+        return self.__show_cover(song)
+
+    def __show_cover(self, song):
+        """Show the cover as a detached BigCenteredImage.
+        If one is already showing, destroy it instead
+        If there is no image, run the AlbumArt plugin
+        """
         if not self.__file and song.is_file:
             from quodlibet.qltk.songsmenu import SongsMenu
             from quodlibet import app
 
             SongsMenu.plugins.handle(ALBUM_ART_PLUGIN_ID, app.library,
                                      qltk.get_top_parent(self), [song])
+
             return True
 
         if self.__current_bci is not None:
